@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2019 The Qt Company Ltd.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
@@ -61,6 +61,10 @@
 
 QT_BEGIN_NAMESPACE
 
+Q_DECLARE_LOGGING_CATEGORY(lcHandlerParent)
+Q_LOGGING_CATEGORY(lcVel, "qt.quick.flickable.velocity")
+Q_LOGGING_CATEGORY(lcWheel, "qt.quick.flickable.wheel")
+
 // FlickThreshold determines how far the "mouse" must have moved
 // before we perform a flick.
 static const int FlickThreshold = 15;
@@ -69,10 +73,6 @@ static const int FlickThreshold = 15;
 // will ensure the Flickable retains the grab on consecutive flicks.
 static const int RetainGrabVelocity = 100;
 
-#ifdef Q_OS_OSX
-static const int MovementEndingTimerInterval = 100;
-#endif
-
 // Currently std::round can't be used on Android when using ndk g++, so
 // use C version instead. We could just define two versions of Round, one
 // for float and one for double, but then only one of them would be used
@@ -130,8 +130,13 @@ void QQuickFlickableVisibleArea::updateVisible()
     // Vertical
     const qreal viewheight = flickable->height();
     const qreal maxyextent = -flickable->maxYExtent() + flickable->minYExtent();
-    qreal pagePos = (-p->vData.move.value() + flickable->minYExtent()) / (maxyextent + viewheight);
-    qreal pageSize = viewheight / (maxyextent + viewheight);
+    const qreal maxYBounds = maxyextent + viewheight;
+    qreal pagePos = 0;
+    qreal pageSize = 0;
+    if (!qFuzzyIsNull(maxYBounds)) {
+        pagePos = (-p->vData.move.value() + flickable->minYExtent()) / maxYBounds;
+        pageSize = viewheight / maxYBounds;
+    }
 
     if (pageSize != m_heightRatio) {
         m_heightRatio = pageSize;
@@ -145,8 +150,14 @@ void QQuickFlickableVisibleArea::updateVisible()
     // Horizontal
     const qreal viewwidth = flickable->width();
     const qreal maxxextent = -flickable->maxXExtent() + flickable->minXExtent();
-    pagePos = (-p->hData.move.value() + flickable->minXExtent()) / (maxxextent + viewwidth);
-    pageSize = viewwidth / (maxxextent + viewwidth);
+    const qreal maxXBounds = maxxextent + viewwidth;
+    if (!qFuzzyIsNull(maxXBounds)) {
+        pagePos = (-p->hData.move.value() + flickable->minXExtent()) / maxXBounds;
+        pageSize = viewwidth / maxXBounds;
+    } else {
+        pagePos = 0;
+        pageSize = 0;
+    }
 
     if (pageSize != m_widthRatio) {
         m_widthRatio = pageSize;
@@ -208,8 +219,8 @@ public:
             axisData->move.setValue(-flickable->contentX());
         else
             axisData->move.setValue(-flickable->contentY());
-        cancel();
         active = false;
+        cancel();
     }
 
 protected:
@@ -248,12 +259,14 @@ QQuickFlickablePrivate::QQuickFlickablePrivate()
     , stealMouse(false), pressed(false)
     , scrollingPhase(false), interactive(true), calcVelocity(false)
     , pixelAligned(false)
+    , syncDrag(false)
     , lastPosTime(-1)
     , lastPressTime(0)
     , deceleration(QML_FLICK_DEFAULTDECELERATION)
     , maxVelocity(QML_FLICK_DEFAULTMAXVELOCITY), reportedVelocitySmoothing(100)
     , delayedPressEvent(nullptr), pressDelay(0), fixupDuration(400)
-    , flickBoost(1.0), fixupMode(Normal), vTime(0), visibleArea(nullptr)
+    , flickBoost(1.0), initialWheelFlickDistance(qApp->styleHints()->wheelScrollLines() * 24)
+    , fixupMode(Normal), vTime(0), visibleArea(nullptr)
     , flickableDirection(QQuickFlickable::AutoFlickDirection)
     , boundsBehavior(QQuickFlickable::DragAndOvershootBounds)
     , boundsMovement(QQuickFlickable::FollowBoundsBehavior)
@@ -267,9 +280,9 @@ void QQuickFlickablePrivate::init()
     QQml_setParent_noEvent(contentItem, q);
     contentItem->setParentItem(q);
     qmlobject_connect(&timeline, QQuickTimeLine, SIGNAL(completed()),
-                      q, QQuickFlickable, SLOT(timelineCompleted()))
+                      q, QQuickFlickable, SLOT(timelineCompleted()));
     qmlobject_connect(&velocityTimeline, QQuickTimeLine, SIGNAL(completed()),
-                      q, QQuickFlickable, SLOT(velocityTimelineCompleted()))
+                      q, QQuickFlickable, SLOT(velocityTimelineCompleted()));
     q->setAcceptedMouseButtons(Qt::LeftButton);
     q->setAcceptTouchEvents(false); // rely on mouse events synthesized from touch
     q->setFiltersChildMouseEvents(true);
@@ -317,7 +330,7 @@ void QQuickFlickablePrivate::itemGeometryChanged(QQuickItem *item, QQuickGeometr
 {
     Q_Q(QQuickFlickable);
     if (item == contentItem) {
-        Qt::Orientations orient = nullptr;
+        Qt::Orientations orient;
         if (change.xChange())
             orient |= Qt::Horizontal;
         if (change.yChange())
@@ -508,7 +521,8 @@ static bool fuzzyLessThanOrEqualTo(qreal a, qreal b)
 void QQuickFlickablePrivate::updateBeginningEnd()
 {
     Q_Q(QQuickFlickable);
-    bool atBoundaryChange = false;
+    bool atXBeginningChange = false, atXEndChange = false;
+    bool atYBeginningChange = false, atYEndChange = false;
 
     // Vertical
     const qreal maxyextent = -q->maxYExtent();
@@ -519,11 +533,15 @@ void QQuickFlickablePrivate::updateBeginningEnd()
 
     if (atBeginning != vData.atBeginning) {
         vData.atBeginning = atBeginning;
-        atBoundaryChange = true;
+        atYBeginningChange = true;
+        if (!vData.moving && atBeginning)
+            vData.smoothVelocity.setValue(0);
     }
     if (atEnd != vData.atEnd) {
         vData.atEnd = atEnd;
-        atBoundaryChange = true;
+        atYEndChange = true;
+        if (!vData.moving && atEnd)
+            vData.smoothVelocity.setValue(0);
     }
 
     // Horizontal
@@ -535,11 +553,15 @@ void QQuickFlickablePrivate::updateBeginningEnd()
 
     if (atBeginning != hData.atBeginning) {
         hData.atBeginning = atBeginning;
-        atBoundaryChange = true;
+        atXBeginningChange = true;
+        if (!hData.moving && atBeginning)
+            hData.smoothVelocity.setValue(0);
     }
     if (atEnd != hData.atEnd) {
         hData.atEnd = atEnd;
-        atBoundaryChange = true;
+        atXEndChange = true;
+        if (!hData.moving && atEnd)
+            hData.smoothVelocity.setValue(0);
     }
 
     if (vData.extentsChanged) {
@@ -560,8 +582,16 @@ void QQuickFlickablePrivate::updateBeginningEnd()
         }
     }
 
-    if (atBoundaryChange)
+    if (atXEndChange || atYEndChange || atXBeginningChange || atYBeginningChange)
         emit q->isAtBoundaryChanged();
+    if (atXEndChange)
+        emit q->atXEndChanged();
+    if (atXBeginningChange)
+        emit q->atXBeginningChanged();
+    if (atYEndChange)
+        emit q->atYEndChanged();
+    if (atYBeginningChange)
+        emit q->atYBeginningChanged();
 
     if (visibleArea)
         visibleArea->updateVisible();
@@ -572,8 +602,6 @@ void QQuickFlickablePrivate::updateBeginningEnd()
 
     This signal is emitted when the view starts to be dragged due to user
     interaction.
-
-    The corresponding handler is \c onDragStarted.
 */
 
 /*!
@@ -583,8 +611,6 @@ void QQuickFlickablePrivate::updateBeginningEnd()
 
     If the velocity of the drag is sufficient at the time the
     touch/mouse button is released then a flick will start.
-
-    The corresponding handler is \c onDragEnded.
 */
 
 /*!
@@ -670,8 +696,6 @@ void QQuickFlickablePrivate::updateBeginningEnd()
 
     This signal is emitted when the view begins moving due to user
     interaction or a generated flick().
-
-    The corresponding handler is \c onMovementStarted.
 */
 
 /*!
@@ -682,8 +706,6 @@ void QQuickFlickablePrivate::updateBeginningEnd()
     be emitted once the flick stops.  If a flick was not
     active, this signal will be emitted when the
     user stops dragging - i.e. a mouse or touch release.
-
-    The corresponding handler is \c onMovementEnded.
 */
 
 /*!
@@ -692,16 +714,12 @@ void QQuickFlickablePrivate::updateBeginningEnd()
     This signal is emitted when the view is flicked.  A flick
     starts from the point that the mouse or touch is released,
     while still in motion.
-
-    The corresponding handler is \c onFlickStarted.
 */
 
 /*!
     \qmlsignal QtQuick::Flickable::flickEnded()
 
     This signal is emitted when the view stops moving due to a flick.
-
-    The corresponding handler is \c onFlickEnded.
 */
 
 /*!
@@ -778,7 +796,7 @@ void QQuickFlickable::setContentX(qreal pos)
     d->hData.vTime = d->timeline.time();
     if (isMoving() || isFlicking())
         movementEnding(true, false);
-    if (-pos != d->hData.move.value())
+    if (!qFuzzyCompare(-pos, d->hData.move.value()))
         d->hData.move.setValue(-pos);
 }
 
@@ -796,7 +814,7 @@ void QQuickFlickable::setContentY(qreal pos)
     d->vData.vTime = d->timeline.time();
     if (isMoving() || isFlicking())
         movementEnding(false, true);
-    if (-pos != d->vData.move.value())
+    if (!qFuzzyCompare(-pos, d->vData.move.value()))
         d->vData.move.setValue(-pos);
 }
 
@@ -984,6 +1002,33 @@ void QQuickFlickable::setPixelAligned(bool align)
     }
 }
 
+/*!
+    \qmlproperty bool QtQuick::Flickable::synchronousDrag
+    \since 5.12
+
+    If this property is set to true, then when the mouse or touchpoint moves
+    far enough to begin dragging the content, the content will jump, such that
+    the content pixel which was under the cursor or touchpoint when pressed
+    remains under that point.
+
+    The default is \c false, which provides a smoother experience (no jump)
+    at the cost that some of the drag distance is "lost" at the beginning.
+*/
+bool QQuickFlickable::synchronousDrag() const
+{
+    Q_D(const QQuickFlickable);
+    return d->syncDrag;
+}
+
+void QQuickFlickable::setSynchronousDrag(bool v)
+{
+    Q_D(QQuickFlickable);
+    if (v != d->syncDrag) {
+        d->syncDrag = v;
+        emit synchronousDragChanged();
+    }
+}
+
 qint64 QQuickFlickablePrivate::computeCurrentTime(QInputEvent *event) const
 {
     if (0 != event->timestamp())
@@ -1100,7 +1145,7 @@ void QQuickFlickablePrivate::drag(qint64 currentTimestamp, QEvent::Type eventTyp
         if (overThreshold || elapsedSincePress > 200) {
             if (!vMoved)
                 vData.dragStartOffset = dy;
-            qreal newY = dy + vData.pressPos - vData.dragStartOffset;
+            qreal newY = dy + vData.pressPos - (syncDrag ? 0 : vData.dragStartOffset);
             // Recalculate bounds in case margins have changed, but use the content
             // size estimate taken at the start of the drag in case the drag causes
             // the estimate to be altered
@@ -1176,7 +1221,7 @@ void QQuickFlickablePrivate::drag(qint64 currentTimestamp, QEvent::Type eventTyp
         if (overThreshold || elapsedSincePress > 200) {
             if (!hMoved)
                 hData.dragStartOffset = dx;
-            qreal newX = dx + hData.pressPos - hData.dragStartOffset;
+            qreal newX = dx + hData.pressPos - (syncDrag ? 0 : hData.dragStartOffset);
             const qreal minX = hData.dragMinBound + hData.startMargin;
             const qreal maxX = hData.dragMaxBound - hData.endMargin;
             if (!(boundsBehavior & QQuickFlickable::DragOverBounds)) {
@@ -1280,7 +1325,7 @@ void QQuickFlickablePrivate::drag(qint64 currentTimestamp, QEvent::Type eventTyp
 void QQuickFlickablePrivate::handleMouseMoveEvent(QMouseEvent *event)
 {
     Q_Q(QQuickFlickable);
-    if (!interactive || lastPosTime == -1)
+    if (!interactive || lastPosTime == -1 || event->buttons() == Qt::NoButton)
         return;
 
     qint64 currentTimestamp = computeCurrentTime(event);
@@ -1390,7 +1435,7 @@ void QQuickFlickablePrivate::handleMouseReleaseEvent(QMouseEvent *event)
 void QQuickFlickable::mousePressEvent(QMouseEvent *event)
 {
     Q_D(QQuickFlickable);
-    if (d->interactive) {
+    if (d->interactive && d->wantsPointerEvent(event)) {
         if (!d->pressed)
             d->handleMousePressEvent(event);
         event->accept();
@@ -1402,7 +1447,7 @@ void QQuickFlickable::mousePressEvent(QMouseEvent *event)
 void QQuickFlickable::mouseMoveEvent(QMouseEvent *event)
 {
     Q_D(QQuickFlickable);
-    if (d->interactive) {
+    if (d->interactive && d->wantsPointerEvent(event)) {
         d->handleMouseMoveEvent(event);
         event->accept();
     } else {
@@ -1413,7 +1458,7 @@ void QQuickFlickable::mouseMoveEvent(QMouseEvent *event)
 void QQuickFlickable::mouseReleaseEvent(QMouseEvent *event)
 {
     Q_D(QQuickFlickable);
-    if (d->interactive) {
+    if (d->interactive && d->wantsPointerEvent(event)) {
         if (d->delayedPressEvent) {
             d->replayDelayedPress();
 
@@ -1441,7 +1486,7 @@ void QQuickFlickable::mouseReleaseEvent(QMouseEvent *event)
 void QQuickFlickable::wheelEvent(QWheelEvent *event)
 {
     Q_D(QQuickFlickable);
-    if (!d->interactive) {
+    if (!d->interactive || !d->wantsPointerEvent(event)) {
         QQuickItem::wheelEvent(event);
         return;
     }
@@ -1454,46 +1499,62 @@ void QQuickFlickable::wheelEvent(QWheelEvent *event)
         d->vData.velocity = 0;
         d->hData.velocity = 0;
         d->timer.start();
-        d->maybeBeginDrag(currentTimestamp, event->posF());
+        d->maybeBeginDrag(currentTimestamp, event->position());
+        d->lastPosTime = -1;
         break;
     case Qt::NoScrollPhase: // default phase with an ordinary wheel mouse
     case Qt::ScrollUpdate:
         if (d->scrollingPhase)
             d->pressed = true;
-#ifdef Q_OS_MACOS
-        // TODO eliminate this timer when ScrollMomentum has been added
-        d->movementEndingTimer.start(MovementEndingTimerInterval, this);
-#endif
+        break;
+    case Qt::ScrollMomentum:
+        d->pressed = false;
+        d->scrollingPhase = false;
+        d->draggingEnding();
+        event->accept();
+        d->lastPosTime = -1;
         break;
     case Qt::ScrollEnd:
-        // TODO most of this should be done at transition to ScrollMomentum phase,
-        // then do what the movementEndingTimer triggers at transition to ScrollEnd phase
         d->pressed = false;
         d->scrollingPhase = false;
         d->draggingEnding();
         event->accept();
         returnToBounds();
         d->lastPosTime = -1;
-#ifdef Q_OS_MACOS
-        d->movementEndingTimer.start(MovementEndingTimerInterval, this);
-#endif
+        d->stealMouse = false;
+        if (!d->velocityTimeline.isActive() && !d->timeline.isActive())
+            movementEnding(true, true);
+        return;
+    }
+
+    qreal elapsed = qreal(currentTimestamp - d->lastPosTime) / qreal(1000);
+    if (elapsed <= 0) {
+        d->lastPosTime = currentTimestamp;
+        qCDebug(lcWheel) << "insufficient elapsed time: can't calculate velocity" << elapsed;
         return;
     }
 
     if (event->source() == Qt::MouseEventNotSynthesized || event->pixelDelta().isNull()) {
-        // physical mouse wheel, so use angleDelta
+        // no pixel delta (physical mouse wheel, or "dumb" touchpad), so use angleDelta
         int xDelta = event->angleDelta().x();
         int yDelta = event->angleDelta().y();
+        // For a single "clicky" wheel event (angleDelta +/- 120),
+        // we want flick() to end up moving a distance proportional to QStyleHints::wheelScrollLines().
+        // The decel algo from there is
+        // qreal dist = v2 / (accel * 2.0);
+        // i.e. initialWheelFlickDistance = (120 / dt)^2 / (deceleration * 2)
+        // now solve for dt:
+        // dt = 120 / sqrt(deceleration * 2 * initialWheelFlickDistance)
+        if (!isMoving())
+            elapsed = 120 / qSqrt(d->deceleration * 2 * d->initialWheelFlickDistance);
         if (yflick() && yDelta != 0) {
-            bool valid = false;
-            if (yDelta > 0 && contentY() > -minYExtent()) {
-                d->vData.velocity = qMax(yDelta*2 - d->vData.smoothVelocity.value(), qreal(d->maxVelocity/4));
-                valid = true;
-            } else if (yDelta < 0 && contentY() < -maxYExtent()) {
-                d->vData.velocity = qMin(yDelta*2 - d->vData.smoothVelocity.value(), qreal(-d->maxVelocity/4));
-                valid = true;
-            }
-            if (valid) {
+            qreal instVelocity = yDelta / elapsed;
+            // if the direction has changed, start over with filtering, to allow instant movement in the opposite direction
+            if ((instVelocity < 0 && d->vData.velocity > 0) || (instVelocity > 0 && d->vData.velocity < 0))
+                d->vData.velocityBuffer.clear();
+            d->vData.addVelocitySample(instVelocity, d->maxVelocity);
+            d->vData.updateVelocity();
+            if ((yDelta > 0 && contentY() > -minYExtent()) || (yDelta < 0 && contentY() < -maxYExtent())) {
                 d->flickY(d->vData.velocity);
                 d->flickingStarted(false, true);
                 if (d->vData.flicking) {
@@ -1504,15 +1565,13 @@ void QQuickFlickable::wheelEvent(QWheelEvent *event)
             }
         }
         if (xflick() && xDelta != 0) {
-            bool valid = false;
-            if (xDelta > 0 && contentX() > -minXExtent()) {
-                d->hData.velocity = qMax(xDelta*2 - d->hData.smoothVelocity.value(), qreal(d->maxVelocity/4));
-                valid = true;
-            } else if (xDelta < 0 && contentX() < -maxXExtent()) {
-                d->hData.velocity = qMin(xDelta*2 - d->hData.smoothVelocity.value(), qreal(-d->maxVelocity/4));
-                valid = true;
-            }
-            if (valid) {
+            qreal instVelocity = xDelta / elapsed;
+            // if the direction has changed, start over with filtering, to allow instant movement in the opposite direction
+            if ((instVelocity < 0 && d->hData.velocity > 0) || (instVelocity > 0 && d->hData.velocity < 0))
+                d->hData.velocityBuffer.clear();
+            d->hData.addVelocitySample(instVelocity, d->maxVelocity);
+            d->hData.updateVelocity();
+            if ((xDelta > 0 && contentX() > -minXExtent()) || (xDelta < 0 && contentX() < -maxXExtent())) {
                 d->flickX(d->hData.velocity);
                 d->flickingStarted(true, false);
                 if (d->hData.flicking) {
@@ -1527,17 +1586,13 @@ void QQuickFlickable::wheelEvent(QWheelEvent *event)
         int xDelta = event->pixelDelta().x();
         int yDelta = event->pixelDelta().y();
 
-        qreal elapsed = qreal(currentTimestamp - d->lastPosTime) / 1000.;
-        if (elapsed <= 0) {
-            d->lastPosTime = currentTimestamp;
-            return;
-        }
         QVector2D velocity(xDelta / elapsed, yDelta / elapsed);
-        d->lastPosTime = currentTimestamp;
         d->accumulatedWheelPixelDelta += QVector2D(event->pixelDelta());
-        d->drag(currentTimestamp, event->type(), event->posF(), d->accumulatedWheelPixelDelta, true, !d->scrollingPhase, true, velocity);
+        d->drag(currentTimestamp, event->type(), event->position(), d->accumulatedWheelPixelDelta,
+                true, !d->scrollingPhase, true, velocity);
         event->accept();
     }
+    d->lastPosTime = currentTimestamp;
 
     if (!event->isAccepted())
         QQuickItem::wheelEvent(event);
@@ -1672,14 +1727,6 @@ void QQuickFlickable::timerEvent(QTimerEvent *event)
         if (d->delayedPressEvent) {
             d->replayDelayedPress();
         }
-    } else if (event->timerId() == d->movementEndingTimer.timerId()) {
-        d->movementEndingTimer.stop();
-        if (!d->scrollingPhase) {
-            d->pressed = false;
-            d->stealMouse = false;
-            if (!d->velocityTimeline.isActive() && !d->timeline.isActive())
-                movementEnding(true, true);
-        }
     }
 }
 
@@ -1716,6 +1763,10 @@ void QQuickFlickable::componentComplete()
         setContentX(-minXExtent());
     if (!d->vData.explicitValue && d->vData.startMargin != 0.)
         setContentY(-minYExtent());
+    if (lcWheel().isDebugEnabled() || lcVel().isDebugEnabled()) {
+        d->timeline.setObjectName(QLatin1String("timeline for Flickable ") + objectName());
+        d->velocityTimeline.setObjectName(QLatin1String("velocity timeline for Flickable ") + objectName());
+    }
 }
 
 void QQuickFlickable::viewportMoved(Qt::Orientations orient)
@@ -1781,7 +1832,7 @@ void QQuickFlickable::geometryChanged(const QRectF &newGeometry,
     if (newGeometry.width() != oldGeometry.width()) {
         changed = true; // we must update visualArea.widthRatio
         if (d->hData.viewSize < 0)
-            d->contentItem->setWidth(width());
+            d->contentItem->setWidth(width() - d->hData.startMargin - d->hData.endMargin);
         // Make sure that we're entirely in view.
         if (!d->pressed && !d->hData.moving && !d->vData.moving) {
             d->fixupMode = QQuickFlickablePrivate::Immediate;
@@ -1791,7 +1842,7 @@ void QQuickFlickable::geometryChanged(const QRectF &newGeometry,
     if (newGeometry.height() != oldGeometry.height()) {
         changed = true; // we must update visualArea.heightRatio
         if (d->vData.viewSize < 0)
-            d->contentItem->setHeight(height());
+            d->contentItem->setHeight(height() - d->vData.startMargin - d->vData.endMargin);
         // Make sure that we're entirely in view.
         if (!d->pressed && !d->hData.moving && !d->vData.moving) {
             d->fixupMode = QQuickFlickablePrivate::Immediate;
@@ -1821,8 +1872,8 @@ void QQuickFlickable::flick(qreal xVelocity, qreal yVelocity)
     d->vData.velocity = yVelocity;
     d->hData.vTime = d->vData.vTime = d->timeline.time();
 
-    bool flickedX = d->flickX(xVelocity);
-    bool flickedY = d->flickY(yVelocity);
+    const bool flickedX = xflick() && !qFuzzyIsNull(xVelocity) && d->flickX(xVelocity);
+    const bool flickedY = yflick() && !qFuzzyIsNull(yVelocity) && d->flickY(yVelocity);
 
     if (flickedX)
         d->hMoved = true;
@@ -1869,8 +1920,13 @@ void QQuickFlickable::cancelFlick()
 
 void QQuickFlickablePrivate::data_append(QQmlListProperty<QObject> *prop, QObject *o)
 {
+    if (!prop || !prop->data)
+        return;
+
     if (QQuickItem *i = qmlobject_cast<QQuickItem *>(o)) {
         i->setParentItem(static_cast<QQuickFlickablePrivate*>(prop->data)->contentItem);
+    } else if (QQuickPointerHandler *pointerHandler = qmlobject_cast<QQuickPointerHandler *>(o)) {
+        static_cast<QQuickFlickablePrivate*>(prop->data)->addPointerHandler(pointerHandler);
     } else {
         o->setParent(prop->object); // XXX todo - do we want this?
     }
@@ -2047,7 +2103,7 @@ void QQuickFlickable::setContentWidth(qreal w)
         return;
     d->hData.viewSize = w;
     if (w < 0)
-        d->contentItem->setWidth(width());
+        d->contentItem->setWidth(width() - d->hData.startMargin - d->hData.endMargin);
     else
         d->contentItem->setWidth(w);
     d->hData.markExtentsDirty();
@@ -2076,7 +2132,7 @@ void QQuickFlickable::setContentHeight(qreal h)
         return;
     d->vData.viewSize = h;
     if (h < 0)
-        d->contentItem->setHeight(height());
+        d->contentItem->setHeight(height() - d->vData.startMargin - d->vData.endMargin);
     else
         d->contentItem->setHeight(h);
     d->vData.markExtentsDirty();
@@ -2288,20 +2344,22 @@ qreal QQuickFlickable::vHeight() const
 bool QQuickFlickable::xflick() const
 {
     Q_D(const QQuickFlickable);
-    if ((d->flickableDirection & QQuickFlickable::AutoFlickIfNeeded) && (vWidth() > width()))
+    const int contentWidthWithMargins = d->contentItem->width() + d->hData.startMargin + d->hData.endMargin;
+    if ((d->flickableDirection & QQuickFlickable::AutoFlickIfNeeded) && (contentWidthWithMargins > width()))
         return true;
     if (d->flickableDirection == QQuickFlickable::AutoFlickDirection)
-        return std::floor(qAbs(vWidth() - width()));
+        return std::floor(qAbs(contentWidthWithMargins - width()));
     return d->flickableDirection & QQuickFlickable::HorizontalFlick;
 }
 
 bool QQuickFlickable::yflick() const
 {
     Q_D(const QQuickFlickable);
-    if ((d->flickableDirection & QQuickFlickable::AutoFlickIfNeeded) && (vHeight() > height()))
+    const int contentHeightWithMargins = d->contentItem->height() + d->vData.startMargin + d->vData.endMargin;
+    if ((d->flickableDirection & QQuickFlickable::AutoFlickIfNeeded) && (contentHeightWithMargins > height()))
         return true;
     if (d->flickableDirection == QQuickFlickable::AutoFlickDirection)
-        return std::floor(qAbs(vHeight() - height()));
+        return std::floor(qAbs(contentHeightWithMargins - height()));
     return d->flickableDirection & QQuickFlickable::VerticalFlick;
 }
 
@@ -2330,6 +2388,14 @@ void QQuickFlickablePrivate::cancelInteraction()
     }
 }
 
+void QQuickFlickablePrivate::addPointerHandler(QQuickPointerHandler *h)
+{
+    Q_Q(const QQuickFlickable);
+    qCDebug(lcHandlerParent) << "reparenting handler" << h << "to contentItem of" << q;
+    h->setParent(contentItem);
+    QQuickItemPrivate::get(contentItem)->addPointerHandler(h);
+}
+
 /*!
     QQuickFlickable::filterMouseEvent checks filtered mouse events and potentially steals them.
 
@@ -2402,7 +2468,20 @@ bool QQuickFlickable::filterMouseEvent(QQuickItem *receiver, QMouseEvent *event)
 bool QQuickFlickable::childMouseEventFilter(QQuickItem *i, QEvent *e)
 {
     Q_D(QQuickFlickable);
-    if (!isVisible() || !isEnabled() || !isInteractive()) {
+    auto wantsPointerEvent_helper = [=]() {
+        bool wants = true;
+        if (e->type() >= QEvent::MouseButtonPress && e->type() <= QEvent::MouseMove) {
+            QMouseEvent *me = static_cast<QMouseEvent*>(e);
+            QPointF itemLocalPos = me->localPos();
+            me->setLocalPos(mapFromItem(i, itemLocalPos));
+            wants = d->wantsPointerEvent(e);
+            // re-localize event back to \a i before returning
+            me->setLocalPos(itemLocalPos);
+        }
+        return wants;
+    };
+
+    if (!isVisible() || !isEnabled() || !isInteractive() || !wantsPointerEvent_helper()) {
         d->cancelInteraction();
         return QQuickItem::childMouseEventFilter(i, e);
     }
@@ -2448,9 +2527,23 @@ void QQuickFlickable::setMaximumFlickVelocity(qreal v)
 
 /*!
     \qmlproperty real QtQuick::Flickable::flickDeceleration
-    This property holds the rate at which a flick will decelerate.
+    This property holds the rate at which a flick will decelerate:
+    the higher the number, the faster it slows down when the user stops
+    flicking via touch, touchpad or mouse wheel. For example 0.0001 is nearly
+    "frictionless", and 10000 feels quite "sticky".
 
-    The default value is platform dependent.
+    The default value is platform dependent. Values of zero or less are not allowed.
+
+    \note For touchpad flicking, some platforms drive Flickable directly by
+    sending QWheelEvents with QWheelEvent::phase() being \c Qt::ScrollMomentum,
+    after the user has released all fingers from the touchpad. In that case,
+    the operating system is controlling the deceleration, and this property has
+    no effect.
+
+    \note For mouse wheel scrolling, and for gesture scrolling on touchpads
+    that do not have a momentum phase, extremely large values of
+    flickDeceleration can make Flickable very resistant to scrolling,
+    especially if \l maximumFlickVelocity is too small.
 */
 qreal QQuickFlickable::flickDeceleration() const
 {
@@ -2463,7 +2556,7 @@ void QQuickFlickable::setFlickDeceleration(qreal deceleration)
     Q_D(QQuickFlickable);
     if (deceleration == d->deceleration)
         return;
-    d->deceleration = deceleration;
+    d->deceleration = qMax(0.001, deceleration);
     emit flickDecelerationChanged();
 }
 
@@ -2540,7 +2633,7 @@ void QQuickFlickablePrivate::draggingStarting()
 void QQuickFlickablePrivate::draggingEnding()
 {
     Q_Q(QQuickFlickable);
-    bool wasDragging = hData.dragging || vData.dragging;
+    const bool wasDragging = hData.dragging || vData.dragging;
     if (hData.dragging) {
         hData.dragging = false;
         emit q->draggingHorizontallyChanged();
@@ -2549,12 +2642,14 @@ void QQuickFlickablePrivate::draggingEnding()
         vData.dragging = false;
         emit q->draggingVerticallyChanged();
     }
-    if (wasDragging && !hData.dragging && !vData.dragging) {
-        emit q->draggingChanged();
-        emit q->dragEnded();
+    if (wasDragging) {
+        if (!hData.dragging && !vData.dragging) {
+            emit q->draggingChanged();
+            emit q->dragEnded();
+        }
+        hData.inRebound = false;
+        vData.inRebound = false;
     }
-    hData.inRebound = false;
-    vData.inRebound = false;
 }
 
 bool QQuickFlickablePrivate::isViewMoving() const
