@@ -3,20 +3,16 @@
 
 /*---
 esid: sec-array.prototype.map
-es5id: 15.4.4.19-5-19
-description: Array.prototype.map - the Arguments object can be used as thisArg
+es5id: 15.4.4.19-5-18
+description: Array.prototype.map - Error object can be used as thisArg
 ---*/
 
-var arg;
+var objError = new RangeError();
 
 function callbackfn(val, idx, obj) {
-  return this === arg;
+  return this === objError;
 }
 
-arg = (function() {
-  return arguments;
-}(1, 2, 3));
-
-var testResult = [11].map(callbackfn, arg);
+var testResult = [11].map(callbackfn, objError);
 
 assert.sameValue(testResult[0], true, 'testResult[0]');
