@@ -2,19 +2,18 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 8.12.1-1_34
+es5id: 8.12.1-1_22
 description: >
-    Properties - [[HasOwnProperty]] (non-configurable, non-enumerable
-    own getter/setter property)
+    Properties - [[HasOwnProperty]] (literal own getter/setter
+    property)
 ---*/
 
-var o = {};
-Object.defineProperty(o, "foo", {
-  get: function() {
+var o = {
+  get foo() {
     return 42;
   },
-  set: function() {;
+  set foo(x) {;
   }
-});
+};
 
 assert(o.hasOwnProperty("foo"), 'o.hasOwnProperty("foo") !== true');
