@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.5.4.20-4-55
+es5id: 15.5.4.20-4-42
 description: >
     String.prototype.trim handles whitepace and lineterminators
-    (\u000A\u000A)
+    (ab\uFEFFc)
 ---*/
 
-assert.sameValue("\u000A\u000A".trim(), "", '"\u000A\u000A".trim()');
+assert.sameValue("ab\uFEFFc".trim(), "ab\uFEFFc", '"ab\uFEFFc".trim()');
